﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.GrammaticalOptimization {

  // symbols are characters, possible nonterminal symbols are : 'A'..'Z'
  // phrases and sentences are strings
  // each symbol is only on element in a string

  public interface IGrammar {
    char SentenceSymbol { get; }
    IEnumerable<char> NonTerminalSymbols { get; }
    IEnumerable<char> TerminalSymbols { get; }
    IEnumerable<char> Symbols { get; }

    IEnumerable<Sequence> GetAlternatives(char nt);
    IEnumerable<Sequence> GetTerminalAlternatives(char nt);
    IEnumerable<Sequence> GetNonTerminalAlternatives(char nt);

    int MinPhraseLength(Sequence phrase);
    int MaxPhraseLength(Sequence phrase);
    Sequence CompleteSentenceRandomly(Random random, Sequence phrase, int maxLen);

    bool IsTerminal(char symbol);
    bool IsTerminal(string phrase);
    bool IsNonTerminal(char symbol);
  }
}
